%macro sasdrugdev_getobjects(sddpath=, recursive=false,typeDef=*,dsname=work._getobjects_,recursiveLevel=);
  %global _sddRC_ _sddMsg_;
  %let _sddRC_=%str(-1);
  %let _sddMsg_=;
  %let _sddRowCount_=%str(-1);
  /* recursiveLevel is a new macro variable to indicate recursion levels, while the parameter resursive is the old
     macro variable to indicate recursion. For backwards compatibility, both flags will be valid.  Recursion will
     be detected in the following manner:
     recursiveLevel = blank indicates the old recursion call.  Recursive will be used to detect the level.  Default is false
     recursiveLevel = 0,1,2 indicates the new recursion call.  Recursive will be ignored and the recursiveLevel will be used
  */

  data &dsname(label="Created by sasdrugdev_getobjects" keep=fullPathName Name sizeString  
        owner type dateTimeString lastModifiedString modifiedby  containingFolderName isContainer);
    attrib returnCode     length=8;
    attrib message        length=$200;
    attrib rowCount       length=8;
    attrib fullPathName   length=$200 label="Full Path Name";
    attrib name           length=$200 label="Name";
    attrib sizeString     length=$200 label="Size";
    attrib type           length=$40  label="Type";
    attrib owner          length=$40 label="Owner";
    attrib dateTimeString length=$25 label="Last Modified Date and Time (GMT)";
    attrib lastModifiedString length=$25 label="Last Modified Date and Time (GMT)";
    attrib modifiedby     length=$40 label="ModifiedBy";
    attrib containingFolderName   length=$200    label="Containing Folder Name";
    attrib isContainer    length=8 label="Is File a Container?";

  
  %if %str(&recursiveLevel) ne %str() %then %do;
    %put NEW recursive method will be used!;
      declare javaobj cmd("com/sas/drugdev/remote/sasmacro/command/GetObjectsCommand", "%bquote(&sddpath)",
         &recursiveLevel,"&typeDef");

  %end;
  %else %do;
     %put OLD recursive method will be used!;
      declare javaobj cmd("com/sas/drugdev/remote/sasmacro/command/GetObjectsCommand", "%bquote(&sddpath)",
              "&recursive","&typeDef");
  %end;
    declare javaobj results("com/sas/drugdev/remote/sasmacro/results/GetNodesResult");
    cmd.callVoidMethod("execute", results);
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    if (returnCode=0) then do;
      results.callIntMethod("getObjectCount", rowCount);
      do row=0 to rowCount - 1;
    	results.callStringMethod("getName", row, name);
        results.callStringMethod("getPath", row, fullPathName);
        results.callStringMethod("getSizeString", row, sizeString);      
        results.callLongMethod("getIsContainer", row, isContainer);
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getOwner", row, owner);
        results.callStringMethod("getLastModifiedDate", row, dateTimeString);
        results.callStringMethod("getLastModifiedBy", row, modifiedby);
        results.callStringMethod("getFolderName", row, containingFolderName);
	    results.callStringMethod("getDateLastModified", row, lastModifiedString);	
        output;
      end;
      put "NOTE: SAS Drug Development Macro: sasdrugdev_getobjects 3.5";
      put "NOTE: SAS Drug Development Macro: * returned a list of objects in %bquote(&sddpath)";
      if (rowCount=0) then
          put "NOTE: SAS Drug Development Macro: * No objects were found.";
      put "NOTE: SAS Drug Development Macro: * " rowCount " records written to dataset &dsname";
    end;
    else do;
      put "ERROR: SAS Drug Development Macro: sasdrugdev_getobjects 3.5";
      put "ERROR: SAS Drug Development Macro: * Return Code: " returnCode;
      put "ERROR: SAS Drug Development Macro: * Tried to list the objects in %bquote(&sddpath)";
      put "ERROR: SAS Drug Development Macro: * " message;
    end;
    call symputx("_sddRC_", returnCode);
    call symputx("_sddMsg_", message);
    call symputx("_sddRowCount_", rowCount);
  run;
  %if &_sddRowCount_ > 0 %then %do;
     data &dsname(drop=date time lastModifiedString);
       set &dsname;
       label dateModified="Last Modified Date (GMT)" 
               timeModified="Last Modified Time (GMT)";
       format dateModified date9. timeModified time8.;
       date=scan(lastModifiedString,1,"T");
       time=scan(lastModifiedString,2,"T");
       dateModified=input(date,yymmdd10.);
       timeModified=input(time,hhmmss.);
       run;
     data temp;
       set &dsname;
       upname=upcase(name);
       upname=translate(upname,'1','_');
     run;
     proc sort data=temp;
       by upname;
     run;
     data &dsname;
       set temp(drop=upname);
     run;
     proc datasets library=work;
       delete temp;
     run;
  %end;
%mend sasdrugdev_getobjects;
