%macro sasdrugdev_sdd_setperms(objID=, cmdClass=, dsname=, callingMacro=, recurse=NONE, keepGroup=,verbose=INFO);
   %global _sddRC_ _sddMsg_;

   %let _sddRC_ = %str(-99);
   %let _sddMsg_ = ;

   %if &verbose eq NONE %then %do;
      %let _errorVerbose_ = 0;
      %let _infoVerbose_  = 0;
   %end;
   %else %if &verbose eq ERROR %then %do;
      %let _errorVerbose_ = 1;
      %let _infoVerbose_  = 0;
   %end;
   %else %if &verbose eq INFO %then %do;
      %let _errorVerbose_ = 1;
      %let _infoVerbose_  = 1;
   %end;

   %let tempds=permtemp;
   proc sort data=&dsname out=&tempds;
      by &objID;
   run;

   %if (%sysfunc(exist(&tempds))=0) %then %return;

   data _null_;
      attrib returnCode length=8
             message    length=$200
             firstErrorMessage length=$200;
      retain errorsFound 0 firstErrorCode firstErrorMessage;
      set &tempds end=eof;
      by &objID;

      if (first.&objID) then do;
         %if %eval(&_infoVerbose_) %then %do;
            put "NOTE: SAS Drug Development Macro: Processing &objid " &objid /;
         %end;
         declare javaobj results("com/sas/drugdev/remote/sasmacro/results/PermissionsResult", trim(left(&objID)),"&keepGroup");
         inputCount=0;
		 link printErrors;
		  if returnCode in (-4,-9,-101,-102) then do;
		     link setReturnCodes;
             stop;
		  end; 
      end;
		 %if &callingMacro = sasdrugdev_setfileperms %then
		 %do ;
		    results.callVoidMethod("setAce", trim(left(ACEName)),canRead,canWrite,canDelete,canManage) ;
		 %end ;
		 %else
		 %do ;
         	results.callVoidMethod("setAce", trim(left(ACEName)),canRead,canWrite,canDelete,canManage,
            	                    canInheritRead,canInheritWrite,canInheritDelete,canInheritManage);
         %end ;
         link printErrors;
         inputCount+1;
		 /* prevent cmd execute method from being called when a problem arises.  That method writes
		    the acl out, which we do not want when we find a bad group */
		 if returnCode in (-9,-101,-102) then do;
		     link setReturnCodes;
             stop;
		 end;

      if (last.&objID) then do;
         if (inputCount<1) then do;
            %if %eval(&_infoVerbose_) %then %do;
               put "WARNING: SAS Drug Development Macro: &callingMacro 3.5";
               put "WARNING: SAS Drug Development Macro: * No valid data for " &objID /;
            %end;
            errorsFound + 1;
         end;
         else do;
            declare javaobj cmd("com/sas/drugdev/remote/sasmacro/command/&cmdClass", trim(left(&objID)),"&recurse");
            cmd.callVoidMethod("execute", results);
            link printErrors;
         end; ** end records found **;
      end;   ** end last.objID **;

      if (eof) then link setReturnCodes;
	return;

	setReturnCodes:
        ** if there were errors, return first error caught  **;
         ** otherwise, assume final return code = ok **;
         if (errorsFound>0) then do;
            call symputx("_sddRC_", firstErrorCode);
            call symputx("_sddMsg_", firstErrorMessage);
         end;
         else do;
            call symputx("_sddRC_", returnCode);
            call symputx("_sddMsg_", message);
         end;
	return;
	printErrors:
            results.callIntMethod("getReturnCode", returnCode);
            results.callStringMethod("getReturnMessage", message);
            if (returnCode>=0) then do;
               %if %eval(&_infoVerbose_) %then %do;
                  put "NOTE: SAS Drug Development Macro: &callingMacro 3.5";
                  put "NOTE: SAS Drug Development Macro: * Updated properties for " &objID /;
               %end;
            end;
            else do;
               errorsFound+1;
               %if %eval(&_errorVerbose_) %then %do;
                  put "ERROR: SAS Drug Development Macro: &callingMacro 3.5";
                  put "ERROR: SAS Drug Development Macro: * Return Code: " returnCode;
                  put "ERROR: SAS Drug Development Macro: * Tried to updated properties for " &objID;
                  put "ERROR: SAS Drug Development Macro: * " message;
               %end;
               if (errorsFound=1) then do;
                  firstErrorCode = returnCode;
                  firstErrorMessage = message;
               end;
            end;
	return;
run;
/**/
%if (%sysfunc(exist(&tempds))>0) %then %do;
   proc datasets library=work nolist;
      delete &tempds;
   quit;; run;;;
%end;

%mend sasdrugdev_sdd_setperms;
