%macro sasdrugdev_sdd_setprops(type=, objID=, cmdClass=, dsname=, callingMacro=);
  %global _sddRC_ _sddMsg_;
  %let _sddRC_=%str(-1);
  %let _sddMsg_=;

  %let tempds=proptemp;
   proc sort data=&dsname out=&tempds;
       by &objID;
   run;
   %if (%sysfunc(exist(&tempds))=0) %then
        %return;

* TODO: Catch the bad dataset before it is processes *;

  data _null_;
    attrib returnCode length=8
            message    length=$200;
    retain errorsFound 0 firstErrorCode firstErrorMessage;

    set &tempds end=eof;
	by &objID;

    if (first.&objID) then do;
            declare javaobj results("com/sas/drugdev/remote/sasmacro/results/PropertiesResult", "%bquote(&type)");
			inputCount=0;
      end;

	  if (name ne "") then
	      do;
	           results.callVoidMethod("setProperty", trim(left(name)), trim(left(value)));
  	           inputCount+1;
		  end;

	if (last.&objID) then do;
		if (inputCount<1) then do;
            put "WARNING: SAS Drug Development Macro: &callingMacro 3.5";
            put "WARNING: SAS Drug Development Macro: * No valid data for " &objID;
			returnCode=-1;
		end;
		else do;
		       put;
		       declare javaobj cmd("com/sas/drugdev/remote/sasmacro/command/&cmdClass", trim(left(&objID)));
		       cmd.callVoidMethod("execute", results);
		       results.callIntMethod("getReturnCode", returnCode);
		       results.callStringMethod("getReturnMessage", message);
			   results.callIntMethod("getPropertiesCount", propertiesProcessed);
			   results.callIntMethod("getProtectedPropsCount", propertiesSkipped);
			   results.callIntMethod("getEditablePropsCount", editablePropertiesUpdated);
			   %if (%upcase(&type)=USER) %then %do;
			   		results.callIntMethod("getUserRequiredCount", requiredPropertiesUpdated);
			   %end;

		       if (returnCode>=0) then do;
		            put "NOTE: SAS Drug Development Macro: &callingMacro 3.5";
		            put "NOTE: SAS Drug Development Macro: * Updated properties for " &objID;
					put "NOTE: SAS Drug Development Macro: * " inputCount " records were processed.";
		            put "NOTE: SAS Drug Development Macro: * " propertiesProcessed " properties were processed.";
		            put "NOTE: SAS Drug Development Macro: * " editablePropertiesUpdated " editable properties were processed.";
					%if (%upcase(&type)=USER) %then %do;
					    put "NOTE: SAS Drug Development Macro: * " requiredPropertiesUpdated " required properties were processed.";
					%end;
		            put "NOTE: SAS Drug Development Macro: * " propertiesSkipped " protected properties were processed, but not updated.";
		        end;
		        else do;
				     errorsFound+1;
				     put "ERROR: SAS Drug Development Macro: &callingMacro 3.5";
		             put "ERROR: SAS Drug Development Macro: * Return Code: " returnCode;
		             put "ERROR: SAS Drug Development Macro: * Tried to updated properties for " &objID;
		             put "ERROR: SAS Drug Development Macro: * " message;
					 if (errorsFound=1) then do;
					      firstErrorCode=returnCode;
						  firstErrorMessage=message;
					 end;
		        end;
		end; ** end records found **;
	end;   ** end last.objID **;
    
	if (eof) then do;
		** if there were errors, return first error caught  **;
        **otherwise, assume final return code = ok **;
        if (errorsFound>0) then do;
           call symputx("_sddRC_",   firstErrorCode);
           call symputx("_sddMsg_", firstErrorMessage);
	    end;
		else do;
           call symputx("_sddRC_", returnCode);
           call symputx("_sddMsg_", message);
		end;
	end;   ** end eof **;

run;

	%if (%sysfunc(exist(&tempds))>0) %then %do;
	    proc datasets library=work nolist;
	         delete &tempds;
	    quit;; run;;;
	%end;

%mend sasdrugdev_sdd_setprops;
